"""
Test that line information is recalculated properly for a frame when it moves
from the middle of the backtrace to a zero index.

This is a regression test for a StackFrame bug, where whether frame is zero or
not depends on an internal field. When LLDB was updating its frame list value
of the field wasn't copied into existing StackFrame instances, so those
StackFrame instances, would use an incorrect line entry evaluation logic in
situations if it was in the middle of the stack frame list (not zeroth), and
then moved to the top position. The difference in logic is that for zeroth
frames line entry is returned for program counter, while for other frame
(except for those that "behave like zeroth") it is for the instruction
preceding PC, as PC points to the next instruction after function call. When
the bug is present, when execution stops at the second breakpoint
SBFrame.GetLineEntry() returns line entry for the previous line, rather than
the one with a breakpoint. Note that this is specific to
SBFrame.GetLineEntry(), SBFrame.GetPCAddress().GetLineEntry() would return
correct entry.

This bug doesn't reproduce through an LLDB interpretator, however it happens
when using API directly, for example in LLDB-MI.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ZerothFrame(TestBase):
    def test(self):
        """
        Test that line information is recalculated properly for a frame when it moves
        from the middle of the backtrace to a zero index.
        """
        self.build()
        self.setTearDownCleanup()

        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        main_dot_c = lldb.SBFileSpec("main.c")
        bp1 = target.BreakpointCreateBySourceRegex(
            "// Set breakpoint 1 here", main_dot_c
        )
        bp2 = target.BreakpointCreateBySourceRegex(
            "// Set breakpoint 2 here", main_dot_c
        )

        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, VALID_PROCESS)

        thread = self.thread()

        if self.TraceOn():
            print("Backtrace at the first breakpoint:")
            for f in thread.frames:
                print(f)

        # Check that we have stopped at correct breakpoint.
        self.assertEqual(
            thread.frame[0].GetLineEntry().GetLine(),
            bp1.GetLocationAtIndex(0).GetAddress().GetLineEntry().GetLine(),
            "LLDB reported incorrect line number.",
        )

        # Important to use SBProcess::Continue() instead of
        # self.runCmd('continue'), because the problem doesn't reproduce with
        # 'continue' command.
        process.Continue()

        if self.TraceOn():
            print("Backtrace at the second breakpoint:")
            for f in thread.frames:
                print(f)
        # Check that we have stopped at the breakpoint
        self.assertEqual(
            thread.frame[0].GetLineEntry().GetLine(),
            bp2.GetLocationAtIndex(0).GetAddress().GetLineEntry().GetLine(),
            "LLDB reported incorrect line number.",
        )
        # Double-check with GetPCAddress()
        self.assertEqual(
            thread.frame[0].GetLineEntry().GetLine(),
            thread.frame[0].GetPCAddress().GetLineEntry().GetLine(),
            "LLDB reported incorrect line number.",
        )



"""
Test lldb-dap launch request.
"""

from lldbsuite.test.decorators import skipIfWindows
import lldbdap_testcase
import os


class TestDAP_launch_cwd(lldbdap_testcase.DAPTestCaseBase):
    """
    Tests the default launch of a simple program with a current working
    directory.
    """

    @skipIfWindows
    def test(self):
        program = self.getBuildArtifact("a.out")
        program_parent_dir = os.path.realpath(os.path.dirname(os.path.dirname(program)))
        self.build_and_launch(program, cwd=program_parent_dir)
        self.continue_to_exit()
        # Now get the STDOUT and verify our program argument is correct
        output = self.get_stdout()
        self.assertTrue(output and len(output) > 0, "expect program output")
        lines = output.splitlines()
        found = False
        for line in lines:
            if line.startswith('cwd = "'):
                quote_path = '"%s"' % (program_parent_dir)
                found = True
                self.assertIn(
                    quote_path,
                    line,
                    "working directory '%s' not in '%s'" % (program_parent_dir, line),
                )
        self.assertTrue(found, "verified program working directory")



# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import transform
from mlir.dialects.transform import debug


def run(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            sequence = transform.SequenceOp(
                transform.FailurePropagationMode.Propagate,
                [],
                transform.AnyOpType.get(),
            )
            with InsertionPoint(sequence.body):
                f(sequence.bodyTarget)
                transform.YieldOp()
        print(module)
    return f


@run
def testDebugEmitParamAsRemark(target):
    i0 = IntegerAttr.get(IntegerType.get_signless(32), 0)
    i0_param = transform.ParamConstantOp(transform.AnyParamType.get(), i0)
    debug.emit_param_as_remark(i0_param)
    debug.emit_param_as_remark(i0_param, anchor=target, message="some text")
    # CHECK-LABEL: TEST: testDebugEmitParamAsRemark
    # CHECK: ^{{.*}}(%[[ARG0:.+]]: !transform.any_op):
    # CHECK: %[[PARAM:.*]] = transform.param.constant
    # CHECK: transform.debug.emit_param_as_remark %[[PARAM]]
    # CHECK: transform.debug.emit_param_as_remark %[[PARAM]]
    # CHECK-SAME: "some text"
    # CHECK-SAME: at %[[ARG0]]


@run
def testDebugEmitRemarkAtOp(target):
    debug.emit_remark_at(target, "some text")
    # CHECK-LABEL: TEST: testDebugEmitRemarkAtOp
    # CHECK: ^{{.*}}(%[[ARG0:.+]]: !transform.any_op):
    # CHECK: transform.debug.emit_remark_at %[[ARG0]], "some text"



"""
Tests std::queue functionality.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestQueue(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    @skipIf(
        compiler="clang",
        compiler_version=[">", "16.0"],
        bugnumber="https://github.com/llvm/llvm-project/issues/68968",
    )
    @skipIf(
        compiler="clang",
        compiler_version=["<", "17.0"],
    )
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.import-std-module true")

        if self.expectedCompiler(["clang"]) and self.expectedCompilerVersion(
            [">", "16.0"]
        ):
            queue_type = "std::queue<C>"
        else:
            queue_type = "std::queue<C, std::deque<C, std::allocator<C> > >"

        size_type = "size_type"
        value_type = "value_type"

        # Test std::queue functionality with a std::deque.
        self.expect_expr(
            "q_deque",
            result_type=queue_type,
            result_children=[ValueCheck(children=[ValueCheck(value="1")])],
        )
        self.expect("expr q_deque.pop()")
        self.expect("expr q_deque.push({4})")
        self.expect_expr("q_deque.size()", result_type=size_type, result_value="1")
        self.expect_expr("q_deque.front()", result_type=value_type)
        self.expect_expr("q_deque.back()", result_type=value_type)
        self.expect_expr("q_deque.front().i", result_type="int", result_value="4")
        self.expect_expr("q_deque.back().i", result_type="int", result_value="4")
        self.expect_expr("q_deque.empty()", result_type="bool", result_value="false")
        self.expect("expr q_deque.pop()")
        self.expect("expr q_deque.emplace(5)")
        self.expect_expr("q_deque.front().i", result_type="int", result_value="5")

        # Test std::queue functionality with a std::list.
        if self.expectedCompiler(["clang"]) and self.expectedCompilerVersion(
            [">", "16.0"]
        ):
            queue_type = "std::queue<C, std::list<C> >"
        else:
            queue_type = "std::queue<C, std::list<C, std::allocator<C> > >"

        self.expect_expr(
            "q_list",
            result_type=queue_type,
            result_children=[ValueCheck(children=[ValueCheck(value="1")])],
        )

        self.expect("expr q_list.pop()")
        self.expect("expr q_list.push({4})")
        self.expect_expr("q_list.size()", result_type=size_type, result_value="1")
        self.expect_expr("q_list.front()", result_type=value_type)
        self.expect_expr("q_list.back()", result_type=value_type)
        self.expect_expr("q_list.front().i", result_type="int", result_value="4")
        self.expect_expr("q_list.back().i", result_type="int", result_value="4")
        self.expect_expr("q_list.empty()", result_type="bool", result_value="false")
        self.expect("expr q_list.pop()")
        self.expect("expr q_list.emplace(5)")
        self.expect_expr("q_list.front().i", result_type="int", result_value="5")



"""
Test stop hook functionality
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestStopHooks(TestBase):
    # If your test case doesn't stress debug info, then
    # set this to true.  That way it won't be run once for
    # each debug info format.
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        full_path = os.path.join(self.getSourceDir(), "main.c")
        self.main_start_line = line_number(full_path, "main()")

    def test_bad_handler(self):
        """Test that we give a good error message when the handler is bad"""
        self.script_setup()
        result = lldb.SBCommandReturnObject()

        # First try the wrong number of args handler:
        command = "target stop-hook add -P stop_hook.bad_handle_stop"
        self.interp.HandleCommand(command, result)
        self.assertFalse(result.Succeeded(), "Set the target stop hook")
        self.assertIn(
            "has unexpected argument count",
            result.GetError(),
            "Got the wrong number of args error",
        )

        # Next the no handler at all handler:
        command = "target stop-hook add -P stop_hook.no_handle_stop"

        self.interp.HandleCommand(command, result)
        self.assertFalse(result.Succeeded(), "Set the target stop hook")
        self.assertIn(
            "Abstract method no_handle_stop.handle_stop not implemented",
            result.GetError(),
            "Got the right error",
        )

    def test_self_deleting(self):
        """Test that we can handle a stop hook that deletes itself"""
        self.script_setup()
        # Run to the first breakpoint before setting the stop hook
        # so we don't have to figure out where it showed up in the new
        # target.
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Stop here first", self.main_source_file
        )

        # Now add our stop hook and register it:
        result = lldb.SBCommandReturnObject()
        command = "target stop-hook add -P stop_hook.self_deleting_stop"
        self.interp.HandleCommand(command, result)
        self.assertCommandReturn(result, f"Added my stop hook: {result.GetError()}")

        result_str = result.GetOutput()
        p = re.compile("Stop hook #([0-9]+) added.")
        m = p.match(result_str)
        current_stop_hook_id = m.group(1)
        command = "command script add -o -f stop_hook.handle_stop_hook_id handle_id"
        self.interp.HandleCommand(command, result)
        self.assertCommandReturn(result, "Added my command")

        command = f"handle_id {current_stop_hook_id}"
        self.interp.HandleCommand(command, result)
        self.assertCommandReturn(result, "Registered my stop ID")

        # Now step the process and make sure the stop hook was deleted.
        thread.StepOver()
        self.interp.HandleCommand("target stop-hook list", result)
        self.assertEqual(result.GetOutput().rstrip(), "No stop hooks.", "Deleted hook")

    def test_stop_hooks_scripted(self):
        """Test that a scripted stop hook works with no specifiers"""
        self.stop_hooks_scripted(5, "-I false")

    def test_stop_hooks_scripted_no_entry(self):
        """Test that a scripted stop hook works with no specifiers"""
        self.stop_hooks_scripted(10)

    def test_stop_hooks_scripted_right_func(self):
        """Test that a scripted stop hook fires when there is a function match"""
        self.stop_hooks_scripted(5, "-I 0 -n step_out_of_me")

    def test_stop_hooks_scripted_wrong_func(self):
        """Test that a scripted stop hook doesn't fire when the function does not match"""
        self.stop_hooks_scripted(0, "-I 0 -n main")

    def test_stop_hooks_scripted_right_lines(self):
        """Test that a scripted stop hook fires when there is a function match"""
        self.stop_hooks_scripted(
            5, "-I 0 -f main.c -l 1 -e %d" % (self.main_start_line)
        )

    def test_stop_hooks_scripted_wrong_lines(self):
        """Test that a scripted stop hook doesn't fire when the function does not match"""
        self.stop_hooks_scripted(
            0, "-I 0 -f main.c -l %d -e 100" % (self.main_start_line)
        )

    def test_stop_hooks_scripted_auto_continue(self):
        """Test that the --auto-continue flag works"""
        self.do_test_auto_continue(False)

    def test_stop_hooks_scripted_return_false(self):
        """Test that the returning False from a stop hook works"""
        self.do_test_auto_continue(True)

    def do_test_auto_continue(self, return_true):
        """Test that auto-continue works."""
        # We set auto-continue to 1 but the stop hook only applies to step_out_of_me,
        # so we should end up stopped in main, having run the expression only once.
        self.script_setup()

        result = lldb.SBCommandReturnObject()

        if return_true:
            command = "target stop-hook add -I 0 -P stop_hook.stop_handler -k increment -v 5 -k return_false -v 1 -n step_out_of_me"
        else:
            command = "target stop-hook add -I 0 -G 1 -P stop_hook.stop_handler -k increment -v 5 -n step_out_of_me"

        self.interp.HandleCommand(command, result)
        self.assertTrue(result.Succeeded(), "Set the target stop hook")

        # First run to main.  If we go straight to the first stop hook hit,
        # run_to_source_breakpoint will fail because we aren't at original breakpoint

        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Stop here first", self.main_source_file
        )

        # Now set the breakpoint on step_out_of_me, and make sure we run the
        # expression, then continue back to main.
        bkpt = target.BreakpointCreateBySourceRegex(
            "Set a breakpoint here and step out", self.main_source_file
        )
        self.assertNotEqual(
            bkpt.GetNumLocations(), 0, "Got breakpoints in step_out_of_me"
        )
        process.Continue()

        var = target.FindFirstGlobalVariable("g_var")
        self.assertTrue(var.IsValid())
        self.assertEqual(var.GetValueAsUnsigned(), 6, "Updated g_var")

        func_name = process.GetSelectedThread().frames[0].GetFunctionName()
        self.assertEqual("main", func_name, "Didn't stop at the expected function.")

    def script_setup(self):
        self.interp = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()

        # Bring in our script file:
        script_name = os.path.join(self.getSourceDir(), "stop_hook.py")
        command = "command script import " + script_name
        self.interp.HandleCommand(command, result)
        self.assertTrue(
            result.Succeeded(), "com scr imp failed: %s" % (result.GetError())
        )

        # set a breakpoint at the end of main to catch our auto-continue tests.
        # Do it in the dummy target so it will get copied to our target even when
        # we don't have a chance to stop.
        dummy_target = self.dbg.GetDummyTarget()
        dummy_target.BreakpointCreateBySourceRegex(
            "return result", self.main_source_file
        )

    def stop_hooks_scripted(self, g_var_value, specifier=None):
        self.script_setup()

        result = lldb.SBCommandReturnObject()

        command = "target stop-hook add -P stop_hook.stop_handler -k increment -v 5 "
        if specifier:
            command += specifier

        self.interp.HandleCommand(command, result)
        self.assertTrue(result.Succeeded(), "Set the target stop hook")
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )
        # At this point we've hit our stop hook so we should have run our expression,
        # which increments g_var by the amount specified by the increment key's value.
        while process.GetState() == lldb.eStateRunning:
            continue

        var = target.FindFirstGlobalVariable("g_var")
        self.assertTrue(var.IsValid())
        self.assertEqual(var.GetValueAsUnsigned(), g_var_value, "Updated g_var")



"""
Test lldb logging.  This test just makes sure logging doesn't crash, and produces some output.
"""


import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class LogTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        super(LogTestCase, self).setUp()
        self.log_file = self.getBuildArtifact("log-file.txt")

    def test_file_writing(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.expect("file " + exe, patterns=["Current executable set to .*a.out"])

        if os.path.exists(self.log_file):
            os.remove(self.log_file)

        # By default, Debugger::EnableLog() will set log options to
        # PREPEND_THREAD_NAME + OPTION_THREADSAFE. We don't want the
        # threadnames here, so we enable just threadsafe (-t).
        self.runCmd("log enable -f '%s' lldb commands" % (self.log_file))

        self.runCmd("command alias bp breakpoint")

        self.runCmd("bp set -n main")

        self.runCmd("bp l")

        self.runCmd("log disable lldb")

        self.assertTrue(os.path.isfile(self.log_file))

        with open(self.log_file, "r") as f:
            log_lines = f.read()
        os.remove(self.log_file)

        self.assertGreater(len(log_lines), 0, "Something was written to the log file.")

    # Check that lldb truncates its log files
    def test_log_truncate(self):
        # put something in our log file
        with open(self.log_file, "w") as f:
            for i in range(1, 1000):
                f.write("bacon\n")

        self.runCmd("log enable -f '%s' lldb commands" % self.log_file)
        self.runCmd("help log")
        self.runCmd("log disable lldb")

        self.assertTrue(os.path.isfile(self.log_file))
        with open(self.log_file, "r") as f:
            contents = f.read()

        # check that it got removed
        self.assertEqual(contents.find("bacon"), -1)

    # Check that lldb can append to a log file
    def test_log_append(self):
        # put something in our log file
        with open(self.log_file, "w") as f:
            f.write("bacon\n")

        self.runCmd("log enable -a -f '%s' lldb commands" % self.log_file)
        self.runCmd("help log")
        self.runCmd("log disable lldb")

        self.assertTrue(os.path.isfile(self.log_file))
        with open(self.log_file, "r") as f:
            contents = f.read()

        # check that it is still there
        self.assertEqual(contents.find("bacon"), 0)

    # Enable all log options and check that nothing crashes.
    @skipIfWindows
    def test_all_log_options(self):
        if os.path.exists(self.log_file):
            os.remove(self.log_file)

        self.runCmd(
            "log enable -v -s -T -p -n -S -F -f '%s' lldb commands" % self.log_file
        )
        self.runCmd("help log")
        self.runCmd("log disable lldb")

        self.assertTrue(os.path.isfile(self.log_file))



# RUN: %PYTHON %s | FileCheck %s


from mlir.ir import *
from mlir.dialects import transform
from mlir.dialects.transform import memref


def run(f):
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            print("\nTEST:", f.__name__)
            f()
        print(module)
    return f


@run
def testMemRefAllocaToAllocOpCompact():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("memref.alloca"),
    )
    with InsertionPoint(sequence.body):
        memref.MemRefAllocaToGlobalOp(sequence.bodyTarget)
        transform.YieldOp()
    # CHECK-LABEL: TEST: testMemRefAllocaToAllocOpCompact
    # CHECK: = transform.memref.alloca_to_global
    # CHECK-SAME: (!transform.op<"memref.alloca">)
    # CHECK-SAME: -> (!transform.any_op, !transform.any_op)


@run
def testMemRefAllocaToAllocOpTyped():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("memref.alloca"),
    )
    with InsertionPoint(sequence.body):
        memref.MemRefAllocaToGlobalOp(
            transform.OperationType.get("memref.get_global"),
            transform.OperationType.get("memref.global"),
            sequence.bodyTarget,
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: testMemRefAllocaToAllocOpTyped
    # CHECK: = transform.memref.alloca_to_global
    # CHECK-SAME: -> (!transform.op<"memref.get_global">, !transform.op<"memref.global">)


@run
def testMemRefMultiBufferOpCompact():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("memref.alloc"),
    )
    with InsertionPoint(sequence.body):
        memref.MemRefMultiBufferOp(sequence.bodyTarget, 4)
        transform.YieldOp()
    # CHECK-LABEL: TEST: testMemRefMultiBufferOpCompact
    # CHECK: = transform.memref.multibuffer
    # CHECK-SAME: factor = 4 : i64
    # CHECK-SAME: (!transform.op<"memref.alloc">) -> !transform.any_op


@run
def testMemRefMultiBufferOpTyped():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("memref.alloc"),
    )
    with InsertionPoint(sequence.body):
        memref.MemRefMultiBufferOp(
            transform.OperationType.get("memref.alloc"), sequence.bodyTarget, 4
        )
        transform.YieldOp()
    # CHECK-LABEL: TEST: testMemRefMultiBufferOpTyped
    # CHECK: = transform.memref.multibuffer
    # CHECK-SAME: factor = 4 : i64
    # CHECK-SAME: (!transform.op<"memref.alloc">) -> !transform.op<"memref.alloc">


@run
def testMemRefMultiBufferOpAttributes():
    sequence = transform.SequenceOp(
        transform.FailurePropagationMode.Propagate,
        [],
        transform.OperationType.get("memref.alloc"),
    )
    with InsertionPoint(sequence.body):
        memref.MemRefMultiBufferOp(sequence.bodyTarget, 4, skip_analysis=True)
        transform.YieldOp()
    # CHECK-LABEL: TEST: testMemRefMultiBufferOpAttributes
    # CHECK: = transform.memref.multibuffer
    # CHECK-SAME: factor = 4 : i64
    # CHECK-SAME: skip_analysis
    # CHECK-SAME: (!transform.op<"memref.alloc">) -> !transform.any_op



"""
Test that you can set breakpoint and hit the C++ language exception breakpoint
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCPPExceptionBreakpoint(TestBase):
    my_var = 10

    @add_test_categories(["pyapi"])
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24538")
    def test_cpp_exception_breakpoint(self):
        """Test setting and hitting the C++ exception breakpoint."""
        self.build()
        self.do_cpp_exception_bkpt()

    @add_test_categories(["pyapi"])
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24538")
    def test_dummy_target_cpp_exception_breakpoint(self):
        """Test setting and hitting the C++ exception breakpoint from dummy target."""
        self.build()
        self.do_dummy_target_cpp_exception_bkpt()

    def setUp(self):
        TestBase.setUp(self)
        self.main_source = "main.c"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    def do_cpp_exception_bkpt(self):
        exe = self.getBuildArtifact("a.out")
        error = lldb.SBError()

        self.target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.target, VALID_TARGET)

        exception_bkpt = self.target.BreakpointCreateForException(
            lldb.eLanguageTypeC_plus_plus, False, True
        )
        self.assertTrue(exception_bkpt.IsValid(), "Created exception breakpoint.")

        process = self.target.LaunchSimple(
            None, None, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        thread_list = lldbutil.get_threads_stopped_at_breakpoint(
            process, exception_bkpt
        )
        self.assertEqual(
            len(thread_list), 1, "One thread stopped at the exception breakpoint."
        )

    def do_dummy_target_cpp_exception_bkpt(self):
        exe = self.getBuildArtifact("a.out")
        error = lldb.SBError()

        dummy_exception_bkpt = self.dbg.GetDummyTarget().BreakpointCreateForException(
            lldb.eLanguageTypeC_plus_plus, False, True
        )
        self.assertTrue(
            dummy_exception_bkpt.IsValid(),
            "Created exception breakpoint in dummy target.",
        )

        self.target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.target, VALID_TARGET)

        exception_bkpt = self.target.GetBreakpointAtIndex(0)
        self.assertTrue(
            exception_bkpt.IsValid(),
            "Target primed with exception breakpoint from dummy target.",
        )

        process = self.target.LaunchSimple(
            None, None, self.get_process_working_directory()
        )
        self.assertTrue(process, PROCESS_IS_VALID)

        thread_list = lldbutil.get_threads_stopped_at_breakpoint(
            process, exception_bkpt
        )
        self.assertEqual(
            len(thread_list), 1, "One thread stopped at the exception breakpoint."
        )



"""
Test lldb's ability to read the Arm TLS register.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


@skipUnlessArch("arm")
@skipUnlessPlatform(["linux"])
class ArmLinuxTLSRegister(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_tls_reg_access(self):
        self.build()
        (_, _, thread, _) = lldbutil.run_to_source_breakpoint(
            self, "// Set breakpoint here.", lldb.SBFileSpec("main.c")
        )
        frame = thread.GetFrameAtIndex(0)

        tpidruro_var = frame.FindVariable("tpidruro")
        self.assertTrue(tpidruro_var.IsValid())

        regs = frame.GetRegisters()
        tls_regs = regs.GetFirstValueByName("Thread Local Storage Registers")
        self.assertTrue(tls_regs.IsValid(), "No TLS registers found.")
        tpidruro_reg = tls_regs.GetChildMemberWithName("tpidruro")
        self.assertTrue(tpidruro_reg.IsValid(), "tpidruro register not found.")

        val = tpidruro_var.GetValueAsUnsigned()
        self.assertEqual(tpidruro_reg.GetValueAsUnsigned(), val)
        self.expect("reg read tp", substrs=[hex(val)])

        self.expect(
            "register write tpidruro 0x1234",
            error=True,
            substrs=["Failed to write register 'tpidruro'"],
        )



"""
Test lldb breakpoint with symlinks/realpath and source-map.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil, lldbplatformutil


class BreakpointTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line_in_main = line_number("main.c", "// Set break point at this line.")
        self.line_in_foo = line_number("real/foo.h", "// Set break point at this line.")
        self.line_in_bar = line_number("real/bar.h", "// Set break point at this line.")
        self.line_in_qux = line_number("real/qux.h", "// Set break point at this line.")
        # disable "There is a running process, kill it and restart?" prompt
        self.runCmd("settings set auto-confirm true")
        self.addTearDownHook(lambda: self.runCmd("settings clear auto-confirm"))

    def buildAndCreateTarget(self):
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

    @skipIf(oslist=["windows"])
    @skipIf(hostoslist=["windows"])
    def test_file_line_breakpoint_realpath_and_source_map(self):
        """Test file/line breakpoint with realpathing and source-mapping."""
        self.buildAndCreateTarget()
        cwd = os.getcwd()

        ######################################################################
        # Baseline
        # --------------------------------------------------------------------
        # Breakpoints should be resolved with paths which are in the line-table.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.c", self.line_in_main, num_expected_locations=1, loc_exact=True
        )
        lldbutil.run_break_set_by_file_and_line(
            self,
            "symlink1/foo.h",
            self.line_in_foo,
            num_expected_locations=1,
            loc_exact=True,
        )
        lldbutil.run_break_set_by_file_and_line(
            self,
            "symlink2/bar.h",
            self.line_in_bar,
            num_expected_locations=1,
            loc_exact=True,
        )
        lldbutil.run_break_set_by_file_and_line(
            self,
            "symlink2/qux.h",
            self.line_in_qux,
            num_expected_locations=1,
            loc_exact=True,
        )

        ######################################################################
        # Symlinked file
        # --------------------------------------------------------------------
        # - `symlink1/foo.h` is a symlink file, pointing at `real/foo.h`
        # - main.c includes `symlink1/foo.h`.
        # - As a result, the line-table contains a support file `(test_base_dir)/symlink1/foo.h`
        # - Setting a breakpoint for `real/foo.h` won't be resolved, because it doesn't match the above path.
        # - Setting a realpath prefix to the current working directory will cause the above support file to be realpath'ed to `(test_base_dir)/real/foo.h`
        # - Now setting a breakpoint for `real/foo.h` will be resolved.
        lldbutil.run_break_set_by_file_and_line(
            self,
            "real/foo.h",
            self.line_in_foo,
            num_expected_locations=0,
            loc_exact=True,
        )
        self.runCmd(f'settings set target.source-realpath-prefixes "{cwd}"')
        lldbutil.run_break_set_by_file_and_line(
            self,
            "real/foo.h",
            self.line_in_foo,
            num_expected_locations=1,
            loc_exact=True,
        )
        # Clear settings so that the test below won't be affected.
        self.runCmd("settings clear target.source-realpath-prefixes")

        ######################################################################
        # Symlinked directory
        # --------------------------------------------------------------------
        # - `symlink2` is a symlink directory, pointing at `real`.
        # - So, `symlink2/bar.h` will be realpath'ed to `real/bar.h`.
        # - main.c includes `symlink2/bar.h`.
        # - As a result, the line-table contains a support file `(test_base_dir)/symlink2/bar.h`
        # - Setting a breakpoint for `real/bar.h` won't be resolved, because it doesn't match the above path.
        # - Setting a realpath prefix to the current working directory will cause the above support file to be realpath'ed to `(test_base_dir)/real/bar.h`
        # - Now setting a breakpoint for `real/bar.h` will be resolved.
        lldbutil.run_break_set_by_file_and_line(
            self,
            "real/bar.h",
            self.line_in_foo,
            num_expected_locations=0,
            loc_exact=True,
        )
        self.runCmd(f'settings set target.source-realpath-prefixes "{cwd}"')
        lldbutil.run_break_set_by_file_and_line(
            self,
            "real/bar.h",
            self.line_in_foo,
            num_expected_locations=1,
            loc_exact=True,
        )
        # Clear settings so that the test below won't be affected.
        self.runCmd("settings clear target.source-realpath-prefixes")

        ######################################################################
        # Symlink + source-map
        # --------------------------------------------------------------------
        # - `symlink2` is a symlink directory, pointing at `real`.
        # - So, `symlink2/qux.h` will be realpath'ed to `real/qux.h`.
        # - main.c includes `symlink2/qux.h`.
        # - As a result, the line-table contains a support file `(test_base_dir)/symlink2/qux.h`
        # - Setting a realpath prefix to the current working directory will cause the above support file to be realpath'ed to `(test_base_dir)/real/qux.h`
        # - Setting a breakpoint for `to-be-mapped/qux.h` won't be resolved, because it doesn't match the above path.
        # - After setting a source-map, setting the same breakpoint will be resolved, because the input path `to-be-mapped/qux.h` is reverse-mapped to `real/qux.h`, which matches the realpath'ed support file.
        lldbutil.run_break_set_by_file_and_line(
            self,
            "real/qux.h",
            self.line_in_foo,
            num_expected_locations=0,
            loc_exact=True,
        )
        self.runCmd(f'settings set target.source-realpath-prefixes "{cwd}"')
        lldbutil.run_break_set_by_file_and_line(
            self,
            "to-be-mapped/qux.h",
            self.line_in_foo,
            num_expected_locations=0,
            loc_exact=True,
        )
        self.runCmd('settings set target.source-map "real" "to-be-mapped"')
        lldbutil.run_break_set_by_file_and_line(
            self,
            "to-be-mapped/qux.h",
            self.line_in_foo,
            num_expected_locations=1,
            loc_exact=True,
        )
        # Clear settings so that the test below won't be affected.
        self.runCmd("settings clear target.source-realpath-prefixes")



"""Test that we get thread names when interrupting a process."""

import time
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestInterruptThreadNames(TestBase):
    @skipUnlessDarwin
    def test_internal_bps_resolved(self):
        self.build()

        source_file = lldb.SBFileSpec("main.c")
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "initial hello", source_file
        )

        thread_start_func_names = [
            "start_wqthread",
            "_pthread_wqthread",
            "_pthread_start",
        ]
        known_module_names = [
            "libsystem_c.dylib",
            "libSystem.B.dylib",
            "libsystem_pthread.dylib",
        ]
        bps = []
        for func in thread_start_func_names:
            for module in known_module_names:
                bps.append(target.BreakpointCreateByName(func, module))
        num_resolved = 0
        for bp in bps:
            num_resolved += bp.GetNumResolvedLocations()
        self.assertGreater(num_resolved, 0)

    @skipUnlessDarwin
    def test_internal_bps_deleted_on_relaunch(self):
        self.build()

        source_file = lldb.SBFileSpec("main.c")
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "initial hello", source_file
        )

        self.runCmd("break list --internal")
        output = self.res.GetOutput()
        self.assertEqual(output.count("thread-creation"), 1)

        process.Kill()
        self.runCmd("run", RUN_SUCCEEDED)

        self.runCmd("break list --internal")
        output = self.res.GetOutput()
        self.assertEqual(output.count("thread-creation"), 1)



"""
Test 'target modules dump pcm-info'.
"""

import os
import shutil
import glob

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    @skipUnlessDarwin
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(self, "return", lldb.SBFileSpec("main.m"))

        mod_cache = self.getBuildArtifact("private-module-cache")
        if os.path.isdir(mod_cache):
            shutil.rmtree(mod_cache)

        self.runCmd(f"settings set symbols.clang-modules-cache-path '{mod_cache}'")

        # Cause lldb to generate a Darwin-*.pcm
        self.runCmd("expression @import Darwin")

        # root/<config-hash>/<module-name>-<modulemap-path-hash>.pcm
        pcm_paths = glob.glob(os.path.join(mod_cache, "*", "Darwin-*.pcm"))
        self.assertEqual(len(pcm_paths), 1, "Expected one Darwin pcm")
        pcm_path = pcm_paths[0]

        self.expect(
            f"target modules dump pcm-info '{pcm_path}'",
            startstr=f"Information for module file '{pcm_path}'",
            substrs=["Module name: Darwin"],
        )



"""
Test lldb data formatter subsystem.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class StdChronoDataFormatterTestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def do_test(self):
        """Test that that file and class static variables display correctly."""
        isNotWindowsHost = lldbplatformutil.getHostPlatform() != "windows"
        (self.target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.cpp", False)
        )

        lldbutil.continue_to_breakpoint(process, bkpt)
        self.expect("frame variable ns", substrs=["ns = 1 ns"])
        self.expect("frame variable us", substrs=["us = 12 µs"])
        self.expect("frame variable ms", substrs=["ms = 123 ms"])
        self.expect("frame variable s", substrs=["s = 1234 s"])
        self.expect("frame variable min", substrs=["min = 12345 min"])
        self.expect("frame variable h", substrs=["h = 123456 h"])

        self.expect("frame variable d", substrs=["d = 654321 days"])
        self.expect("frame variable w", substrs=["w = 54321 weeks"])
        self.expect("frame variable m", substrs=["m = 4321 months"])
        self.expect("frame variable y", substrs=["y = 321 years"])

        self.expect(
            "frame variable ss_tp",
            substrs=["ss_tp = date/time=1970-01-01T00:00:00Z timestamp=0 s"],
        )
        self.expect(
            "frame variable ss_tp_d",
            substrs=["ss_tp_d = date/time=1970-01-01T00:00:00Z timestamp=0 s"],
        )
        self.expect(
            "frame variable ss_tp_d_r",
            substrs=["ss_tp_d_r = date/time=1970-01-01T00:00:00Z timestamp=0 s"],
        )
        self.expect(
            "frame variable ss_tp_d_r2",
            substrs=["ss_tp_d_r2 = date/time=1970-01-01T00:00:00Z timestamp=0 s"],
        )

        self.expect(
            "frame variable ss_0",
            substrs=["ss_0 = date/time=1970-01-01T00:00:00Z timestamp=0 s"],
        )

        self.expect(
            "frame variable ss_neg_date_time",
            substrs=[
                (
                    "ss_neg_date_time = date/time=-32767-01-01T00:00:00Z timestamp=-1096193779200 s"
                    if isNotWindowsHost
                    else "ss_neg_date_time = timestamp=-1096193779200 s"
                )
            ],
        )
        self.expect(
            "frame variable ss_neg_seconds",
            substrs=["ss_neg_seconds = timestamp=-1096193779201 s"],
        )

        self.expect(
            "frame variable ss_pos_date_time",
            substrs=[
                (
                    "ss_pos_date_time = date/time=32767-12-31T23:59:59Z timestamp=971890963199 s"
                    if isNotWindowsHost
                    else "ss_pos_date_time = timestamp=971890963199 s"
                )
            ],
        )
        self.expect(
            "frame variable ss_pos_seconds",
            substrs=["ss_pos_seconds = timestamp=971890963200 s"],
        )

        self.expect(
            "frame variable ss_min",
            substrs=["ss_min = timestamp=-9223372036854775808 s"],
        )
        self.expect(
            "frame variable ss_max",
            substrs=["ss_max = timestamp=9223372036854775807 s"],
        )

        self.expect(
            "frame variable sd_tp",
            substrs=["sd_tp = date=1970-01-01Z timestamp=0 days"],
        )
        self.expect(
            "frame variable sd_tp_d_r",
            substrs=["sd_tp_d_r = date=1970-01-01Z timestamp=0 days"],
        )
        self.expect(
            "frame variable sd_tp_d_r2",
            substrs=["sd_tp_d_r2 = date=1970-01-01Z timestamp=0 days"],
        )

        self.expect(
            "frame variable sd_0", substrs=["sd_0 = date=1970-01-01Z timestamp=0 days"]
        )
        self.expect(
            "frame variable sd_neg_date",
            substrs=[
                (
                    "sd_neg_date = date=-32767-01-01Z timestamp=-12687428 days"
                    if isNotWindowsHost
                    else "sd_neg_date = timestamp=-12687428 days"
                )
            ],
        )
        self.expect(
            "frame variable sd_neg_days",
            substrs=["sd_neg_days = timestamp=-12687429 days"],
        )

        self.expect(
            "frame variable sd_pos_date",
            substrs=[
                (
                    "sd_pos_date = date=32767-12-31Z timestamp=11248737 days"
                    if isNotWindowsHost
                    else "sd_pos_date = timestamp=11248737 days"
                )
            ],
        )
        self.expect(
            "frame variable sd_pos_days",
            substrs=["sd_pos_days = timestamp=11248738 days"],
        )

        self.expect(
            "frame variable sd_min",
            substrs=["sd_min = timestamp=-2147483648 days"],
        )
        self.expect(
            "frame variable sd_max",
            substrs=["sd_max = timestamp=2147483647 days"],
        )

        # local_seconds aliasses

        self.expect(
            "frame variable ls_tp",
            substrs=["ls_tp = date/time=1970-01-01T00:00:00 timestamp=0 s"],
        )
        self.expect(
            "frame variable ls_tp_d",
            substrs=["ls_tp_d = date/time=1970-01-01T00:00:00 timestamp=0 s"],
        )
        self.expect(
            "frame variable ls_tp_d_r",
            substrs=["ls_tp_d_r = date/time=1970-01-01T00:00:00 timestamp=0 s"],
        )
        self.expect(
            "frame variable ls_tp_d_r2",
            substrs=["ls_tp_d_r2 = date/time=1970-01-01T00:00:00 timestamp=0 s"],
        )

        # local_seconds

        self.expect(
            "frame variable ls_0",
            substrs=["ls_0 = date/time=1970-01-01T00:00:00 timestamp=0 s"],
        )

        self.expect(
            "frame variable ls_neg_date_time",
            substrs=[
                (
                    "ls_neg_date_time = date/time=-32767-01-01T00:00:00 timestamp=-1096193779200 s"
                    if isNotWindowsHost
                    else "ls_neg_date_time = timestamp=-1096193779200 s"
                )
            ],
        )
        self.expect(
            "frame variable ls_neg_seconds",
            substrs=["ls_neg_seconds = timestamp=-1096193779201 s"],
        )

        self.expect(
            "frame variable ls_pos_date_time",
            substrs=[
                (
                    "ls_pos_date_time = date/time=32767-12-31T23:59:59 timestamp=971890963199 s"
                    if isNotWindowsHost
                    else "ls_pos_date_time = timestamp=971890963199 s"
                )
            ],
        )
        self.expect(
            "frame variable ls_pos_seconds",
            substrs=["ls_pos_seconds = timestamp=971890963200 s"],
        )

        self.expect(
            "frame variable ls_min",
            substrs=["ls_min = timestamp=-9223372036854775808 s"],
        )
        self.expect(
            "frame variable ls_max",
            substrs=["ls_max = timestamp=9223372036854775807 s"],
        )

        # local_days aliasses

        self.expect(
            "frame variable ld_tp",
            substrs=["ld_tp = date=1970-01-01 timestamp=0 days"],
        )
        self.expect(
            "frame variable ld_tp_d_r",
            substrs=["ld_tp_d_r = date=1970-01-01 timestamp=0 days"],
        )
        self.expect(
            "frame variable ld_tp_d_r2",
            substrs=["ld_tp_d_r2 = date=1970-01-01 timestamp=0 days"],
        )

        # local_days

        self.expect(
            "frame variable ld_0", substrs=["ld_0 = date=1970-01-01 timestamp=0 days"]
        )
        self.expect(
            "frame variable ld_neg_date",
            substrs=[
                (
                    "ld_neg_date = date=-32767-01-01 timestamp=-12687428 days"
                    if isNotWindowsHost
                    else "ld_neg_date = timestamp=-12687428 days"
                )
            ],
        )
        self.expect(
            "frame variable ld_neg_days",
            substrs=["ld_neg_days = timestamp=-12687429 days"],
        )

        self.expect(
            "frame variable ld_pos_date",
            substrs=[
                (
                    "ld_pos_date = date=32767-12-31 timestamp=11248737 days"
                    if isNotWindowsHost
                    else "ld_pos_date = timestamp=11248737 days"
                )
            ],
        )
        self.expect(
            "frame variable ld_pos_days",
            substrs=["ld_pos_days = timestamp=11248738 days"],
        )

        self.expect(
            "frame variable ld_min",
            substrs=["ld_min = timestamp=-2147483648 days"],
        )
        self.expect(
            "frame variable ld_max",
            substrs=["ld_max = timestamp=2147483647 days"],
        )

        self.expect("frame variable d_0", substrs=["d_0 = day=0"])
        self.expect("frame variable d_1", substrs=["d_1 = day=1"])
        self.expect("frame variable d_31", substrs=["d_31 = day=31"])
        self.expect("frame variable d_255", substrs=["d_255 = day=255"])

        self.expect("frame variable jan", substrs=["jan = month=January"])
        self.expect("frame variable feb", substrs=["feb = month=February"])
        self.expect("frame variable mar", substrs=["mar = month=March"])
        self.expect("frame variable apr", substrs=["apr = month=April"])
        self.expect("frame variable may", substrs=["may = month=May"])
        self.expect("frame variable jun", substrs=["jun = month=June"])
        self.expect("frame variable jul", substrs=["jul = month=July"])
        self.expect("frame variable aug", substrs=["aug = month=August"])
        self.expect("frame variable sep", substrs=["sep = month=September"])
        self.expect("frame variable oct", substrs=["oct = month=October"])
        self.expect("frame variable nov", substrs=["nov = month=November"])
        self.expect("frame variable dec", substrs=["dec = month=December"])

        self.expect("frame variable month_0", substrs=["month_0 = month=0"])
        self.expect("frame variable month_1", substrs=["month_1 = month=January"])
        self.expect("frame variable month_2", substrs=["month_2 = month=February"])
        self.expect("frame variable month_3", substrs=["month_3 = month=March"])
        self.expect("frame variable month_4", substrs=["month_4 = month=April"])
        self.expect("frame variable month_5", substrs=["month_5 = month=May"])
        self.expect("frame variable month_6", substrs=["month_6 = month=June"])
        self.expect("frame variable month_7", substrs=["month_7 = month=July"])
        self.expect("frame variable month_8", substrs=["month_8 = month=August"])
        self.expect("frame variable month_9", substrs=["month_9 = month=September"])
        self.expect("frame variable month_10", substrs=["month_10 = month=October"])
        self.expect("frame variable month_11", substrs=["month_11 = month=November"])
        self.expect("frame variable month_12", substrs=["month_12 = month=December"])
        self.expect("frame variable month_13", substrs=["month_13 = month=13"])
        self.expect("frame variable month_255", substrs=["month_255 = month=255"])

        self.expect("frame variable sun", substrs=["sun = weekday=Sunday"])
        self.expect("frame variable mon", substrs=["mon = weekday=Monday"])
        self.expect("frame variable tue", substrs=["tue = weekday=Tuesday"])
        self.expect("frame variable wed", substrs=["wed = weekday=Wednesday"])
        self.expect("frame variable thu", substrs=["thu = weekday=Thursday"])
        self.expect("frame variable fri", substrs=["fri = weekday=Friday"])
        self.expect("frame variable sat", substrs=["sat = weekday=Saturday"])

        self.expect("frame variable weekday_0", substrs=["weekday_0 = weekday=Sunday"])
        self.expect("frame variable weekday_1", substrs=["weekday_1 = weekday=Monday"])
        self.expect("frame variable weekday_2", substrs=["weekday_2 = weekday=Tuesday"])
        self.expect(
            "frame variable weekday_3", substrs=["weekday_3 = weekday=Wednesday"]
        )
        self.expect(
            "frame variable weekday_4", substrs=["weekday_4 = weekday=Thursday"]
        )
        self.expect("frame variable weekday_5", substrs=["weekday_5 = weekday=Friday"])
        self.expect(
            "frame variable weekday_6", substrs=["weekday_6 = weekday=Saturday"]
        )
        self.expect("frame variable weekday_7", substrs=["weekday_7 = weekday=Sunday"])
        self.expect("frame variable weekday_8", substrs=["weekday_8 = weekday=8"])
        self.expect("frame variable weekday_255", substrs=["weekday_255 = weekday=255"])

        self.expect(
            "frame variable wdi_saturday_0",
            substrs=["wdi_saturday_0 = weekday=Saturday index=0"],
        )
        self.expect(
            "frame variable wdi_monday_1",
            substrs=["wdi_monday_1 = weekday=Monday index=1"],
        )
        self.expect(
            "frame variable wdi_invalid",
            substrs=["wdi_invalid = weekday=255 index=255"],
        )

        self.expect(
            "frame variable wdl_monday",
            substrs=["wdl_monday = weekday=Monday index=last"],
        )
        self.expect(
            "frame variable wdl_invalid",
            substrs=["wdl_invalid = weekday=255 index=last"],
        )

        self.expect("frame variable y_min", substrs=["y_min = year=-32767"])
        self.expect("frame variable y_0", substrs=["y_0 = year=0"])
        self.expect("frame variable y_1970", substrs=["y_1970 = year=1970"])
        self.expect("frame variable y_2038", substrs=["y_2038 = year=2038"])
        self.expect("frame variable y_max", substrs=["y_max = year=32767"])

        self.expect(
            "frame variable md_new_years_eve",
            substrs=["md_new_years_eve = month=December day=31"],
        )
        self.expect(
            "frame variable md_new_year", substrs=["md_new_year = month=January day=1"]
        )
        self.expect(
            "frame variable md_invalid", substrs=["md_invalid = month=255 day=255"]
        )

        self.expect(
            "frame variable mdl_jan", substrs=["mdl_jan = month=January day=last"]
        )
        self.expect(
            "frame variable mdl_new_years_eve",
            substrs=["mdl_new_years_eve = month=December day=last"],
        )

        self.expect(
            "frame variable mwd_first_thursday",
            substrs=["mwd_first_thursday = month=January weekday=Thursday index=1"],
        )

        self.expect(
            "frame variable mwdl_last_saturday",
            substrs=["mwdl_last_saturday = month=December weekday=Saturday index=last"],
        )

        self.expect(
            "frame variable ym_year_zero",
            substrs=["ym_year_zero = year=0 month=January"],
        )

        self.expect("frame variable ymd_bc", substrs=["ymd_bc = date=-0001-03-255"])
        self.expect(
            "frame variable ymd_year_zero", substrs=["ymd_year_zero = date=0000-255-25"]
        )
        self.expect(
            "frame variable ymd_unix_epoch",
            substrs=["ymd_unix_epoch = date=1970-01-01"],
        )

        self.expect(
            "frame variable ymdl_bc",
            substrs=["ymdl_bc = year=-1 month=December day=last"],
        )
        self.expect(
            "frame variable ymdl_may_1970",
            substrs=["ymdl_may_1970 = year=1970 month=May day=last"],
        )

        self.expect(
            "frame variable ymwd_bc",
            substrs=["ymwd_bc = year=-1 month=June weekday=Wednesday index=2"],
        )
        self.expect(
            "frame variable ymwd_forth_tuesday_2024",
            substrs=[
                "ymwd_forth_tuesday_2024 = year=2024 month=January weekday=Tuesday index=4"
            ],
        )

        self.expect(
            "frame variable ymwdl_bc",
            substrs=["ymwdl_bc = year=-1 month=April weekday=Friday index=last"],
        )
        self.expect(
            "frame variable ymwdl_2024_last_tuesday_january",
            substrs=[
                "ymwdl_2024_last_tuesday_january = year=2024 month=January weekday=Tuesday index=last"
            ],
        )

    @skipIf(compiler="clang", compiler_version=["<", "17.0"])
    @add_test_categories(["libc++"])
    def test_libcxx(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test()



"""
Test evaluating expressions when debugging core file.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


@skipIfLLVMTargetMissing("X86")
class CoreExprTestCase(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.target = self.dbg.CreateTarget("linux-x86_64.out")
        self.process = self.target.LoadCore("linux-x86_64.core")
        self.assertTrue(self.process, PROCESS_IS_VALID)

    def test_result_var(self):
        """Test that the result variable can be used in subsequent expressions."""

        self.expect_expr(
            "outer",
            result_type="Outer",
            result_children=[ValueCheck(name="inner", type="Inner")],
        )
        self.expect_expr(
            "$0.inner",
            result_type="Inner",
            result_children=[ValueCheck(name="val", type="int", value="5")],
        )
        self.expect_expr("$1.val", result_type="int", result_value="5")

    def test_persist_var(self):
        """Test that user-defined variables can be used in subsequent expressions."""

        self.target.EvaluateExpression("int $my_int = 5")
        self.expect_expr("$my_int * 2", result_type="int", result_value="10")

        # Try assigning the persistent variable a new value.
        self.target.EvaluateExpression("$my_int = 55")
        self.expect_expr("$my_int", result_type="int", result_value="55")

    def test_context_object(self):
        """Test expression evaluation in context of an object."""

        val_outer = self.expect_expr("outer", result_type="Outer")

        val_inner = val_outer.EvaluateExpression("inner")
        self.assertTrue(val_inner.IsValid())
        self.assertEqual("Inner", val_inner.GetDisplayTypeName())

        val_val = val_inner.EvaluateExpression("this->val")
        self.assertTrue(val_val.IsValid())
        self.assertEqual("int", val_val.GetDisplayTypeName())
        self.assertEqual(val_val.GetValueAsSigned(), 5)



import lldb


class wrapPtrSynthProvider:
    def __init__(self, valobj, dict):
        self.valobj = valobj

    def num_children(self):
        return 1

    def get_child_at_index(self, index):
        if index == 0:
            return self.valobj.GetChildMemberWithName("ptr")
        if index == 1:
            internal_child = self.valobj.GetChildMemberWithName("ptr")
            if not internal_child:
                return None
            return internal_child.Dereference()
        return None

    def get_child_index(self, name):
        if name == "ptr":
            return 0
        if name == "$$dereference$$":
            return 1
        return -1

    def update(self):
        return True



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExprBug35310(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        self.main_source = "main.cpp"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    def test_issue35310(self):
        """Test invoking functions with non-standard linkage names.

        The GNU abi_tag extension used by libstdc++ is a common source
        of these, but they could originate from other reasons as well.
        """
        self.build()

        lldbutil.run_to_source_breakpoint(self, "// Break here", self.main_source_spec)

        self.expect_expr("a.test_abi_tag()", result_value="1")
        self.expect_expr("a.test_asm_name()", result_value="2")



"""
Test lldb-dap launch request.
"""

import lldbdap_testcase
import os


class TestDAP_launch_sourcePath(lldbdap_testcase.DAPTestCaseBase):
    """
    Tests the "sourcePath" will set the target.source-map.
    """

    def test(self):
        program = self.getBuildArtifact("a.out")
        program_dir = os.path.dirname(program)
        self.build_and_launch(program, sourcePath=program_dir)
        self.continue_to_exit()
        output = self.get_console()
        self.assertTrue(output and len(output) > 0, "expect console output")
        lines = output.splitlines()
        prefix = '(lldb) settings set target.source-map "." '
        found = False
        for line in lines:
            if line.startswith(prefix):
                found = True
                quoted_path = '"%s"' % (program_dir)
                self.assertEqual(
                    quoted_path,
                    line[len(prefix) :],
                    "lldb-dap working dir %s == %s" % (quoted_path, line[6:]),
                )
        self.assertTrue(found, 'found "sourcePath" in console output')



"""
Test lldb's ability to read and write the AArch64 FPMR register.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class AArch64LinuxFPMR(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    # The value set by the inferior.
    EXPECTED_FPMR = (0b101010 << 32) | 0b101
    EXPECTED_FPMR_FIELDS = ["LSCALE2 = 42", "F8S1 = FP8_E4M3 | 0x4"]

    @skipUnlessArch("aarch64")
    @skipUnlessPlatform(["linux"])
    def test_fpmr_register_live(self):
        if not self.isAArch64FPMR():
            self.skipTest("FPMR must be present.")

        self.build()
        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            "main.c",
            line_number("main.c", "// Set break point at this line."),
            num_expected_locations=1,
        )

        self.runCmd("run", RUN_SUCCEEDED)

        if self.process().GetState() == lldb.eStateExited:
            self.fail("Test program failed to run.")

        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # This has been set by the program.
        self.expect(
            "register read --all",
            substrs=[
                "Floating Point Mode Register",
                f"fpmr = {self.EXPECTED_FPMR:#018x}",
            ],
        )

        if self.hasXMLSupport():
            self.expect("register read fpmr", substrs=self.EXPECTED_FPMR_FIELDS)

        # Write a value for the program to find. Same fields but with bit values
        # inverted.
        new_fpmr = (0b010101 << 32) | 0b010
        self.runCmd(f"register write fpmr {new_fpmr:#x}")

        # This value should be saved and restored after expressions.
        self.runCmd("p expr_func()")
        self.expect("register read fpmr", substrs=[f"fpmr = {new_fpmr:#018x}"])

        # 0 means the program found the new value in the sysreg as expected.
        self.expect("continue", substrs=["exited with status = 0"])

    @skipIfLLVMTargetMissing("AArch64")
    def test_fpmr_register_core(self):
        if not self.isAArch64FPMR():
            self.skipTest("FPMR must be present.")

        self.runCmd("target create --core corefile")

        self.expect(
            "register read --all",
            substrs=[
                "Floating Point Mode Register",
                f"fpmr = {self.EXPECTED_FPMR:#018x}",
            ],
        )
        self.expect("register read fpmr", substrs=self.EXPECTED_FPMR_FIELDS)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestBreakInLoadedDylib(TestBase):
    """Test that we can set a source regex breakpoint that will take in
    a dlopened library that hasn't loaded when we set the breakpoint."""

    NO_DEBUG_INFO_TESTCASE = True

    @skipIfRemote
    def common_setup(self):
        self.build()
        ctx = self.platformContext
        self.main_spec = lldb.SBFileSpec("main.cpp")
        self.b_spec = lldb.SBFileSpec("b.cpp")
        self.lib_shortname = "lib_b"
        self.lib_fullname = ctx.getFullLibName(self.lib_shortname)
        self.lib_spec = lldb.SBFileSpec(self.lib_fullname)

    def test_break_in_dlopen_dylib_using_lldbutils(self):
        self.common_setup()
        lldbutil.run_to_source_breakpoint(
            self,
            "Break here in dylib",
            self.b_spec,
            bkpt_module=self.lib_fullname,
            extra_images=[self.lib_shortname],
            has_locations_before_run=False,
        )

    @skipIfRemote
    def test_break_in_dlopen_dylib_using_target(self):
        self.common_setup()

        target, process, _, _ = lldbutil.run_to_source_breakpoint(
            self,
            "Break here before we dlopen",
            self.main_spec,
            extra_images=[self.lib_shortname],
        )

        # Now set some breakpoints that won't take till the library is loaded:
        # This one is currently how lldbutils does it but test here in case that changes:
        bkpt1 = target.BreakpointCreateBySourceRegex(
            "Break here in dylib", self.b_spec, self.lib_fullname
        )

        # Try the file list API as well.  Put in some bogus entries too, to make sure those
        # don't trip us up:

        files_list = lldb.SBFileSpecList()
        files_list.Append(self.b_spec)
        files_list.Append(self.main_spec)
        files_list.Append(lldb.SBFileSpec("I_bet_nobody_has_this_file.cpp"))

        modules_list = lldb.SBFileSpecList()
        modules_list.Append(self.lib_spec)
        modules_list.Append(lldb.SBFileSpec("libI_bet_not_this_one_either.dylib"))

        bkpt2 = target.BreakpointCreateBySourceRegex(
            "Break here in dylib", modules_list, files_list
        )

        lldbutil.continue_to_breakpoint(process, bkpt1)
        self.assertEqual(bkpt1.GetHitCount(), 1, "Hit breakpoint 1")
        self.assertEqual(bkpt2.GetHitCount(), 1, "Hit breakpoint 2")



# RUN: %PYTHON %s | FileCheck %s
import sys
import typing
from typing import Union, Optional

from mlir.ir import *
import mlir.dialects.func as func
import mlir.dialects.python_test as test
import mlir.dialects.tensor as tensor
import mlir.dialects.arith as arith

from mlir._mlir_libs._mlirPythonTestNanobind import (
    TestAttr,
    TestType,
    TestTensorValue,
    TestIntegerRankedTensorType,
)

test.register_python_test_dialect(get_dialect_registry())


def run(f):
    print("\nTEST:", f.__name__)
    f()
    return f


# CHECK-LABEL: TEST: testAttributes
@run
def testAttributes():
    with Context() as ctx, Location.unknown():
        #
        # Check op construction with attributes.
        #

        i32 = IntegerType.get_signless(32)
        one = IntegerAttr.get(i32, 1)
        two = IntegerAttr.get(i32, 2)
        unit = UnitAttr.get()

        # CHECK: python_test.attributed_op  {
        # CHECK-DAG: mandatory_i32 = 1 : i32
        # CHECK-DAG: optional_i32 = 2 : i32
        # CHECK-DAG: unit
        # CHECK: }
        op = test.AttributedOp(one, optional_i32=two, unit=unit)
        print(f"{op}")

        # CHECK: python_test.attributed_op  {
        # CHECK: mandatory_i32 = 2 : i32
        # CHECK: }
        op2 = test.AttributedOp(two)
        print(f"{op2}")

        #
        # Check generic "attributes" access and mutation.
        #

        assert "additional" not in op.attributes

        # CHECK: python_test.attributed_op  {
        # CHECK-DAG: additional = 1 : i32
        # CHECK-DAG: mandatory_i32 = 2 : i32
        # CHECK: }
        op2.attributes["additional"] = one
        print(f"{op2}")

        # CHECK: python_test.attributed_op  {
        # CHECK-DAG: additional = 2 : i32
        # CHECK-DAG: mandatory_i32 = 2 : i32
        # CHECK: }
        op2.attributes["additional"] = two
        print(f"{op2}")

        # CHECK: python_test.attributed_op  {
        # CHECK-NOT: additional = 2 : i32
        # CHECK:     mandatory_i32 = 2 : i32
        # CHECK: }
        del op2.attributes["additional"]
        print(f"{op2}")

        try:
            print(op.attributes["additional"])
        except KeyError:
            pass
        else:
            assert False, "expected KeyError on unknown attribute key"

        #
        # Check accessors to defined attributes.
        #

        # CHECK: Mandatory: 1
        # CHECK: Optional: 2
        # CHECK: Unit: True
        print(f"Mandatory: {op.mandatory_i32.value}")
        print(f"Optional: {op.optional_i32.value}")
        print(f"Unit: {op.unit}")

        # CHECK: Mandatory: 2
        # CHECK: Optional: None
        # CHECK: Unit: False
        print(f"Mandatory: {op2.mandatory_i32.value}")
        print(f"Optional: {op2.optional_i32}")
        print(f"Unit: {op2.unit}")

        # CHECK: Mandatory: 2
        # CHECK: Optional: None
        # CHECK: Unit: False
        op.mandatory_i32 = two
        op.optional_i32 = None
        op.unit = False
        print(f"Mandatory: {op.mandatory_i32.value}")
        print(f"Optional: {op.optional_i32}")
        print(f"Unit: {op.unit}")
        assert "optional_i32" not in op.attributes
        assert "unit" not in op.attributes

        try:
            op.mandatory_i32 = None
        except ValueError:
            pass
        else:
            assert False, "expected ValueError on setting a mandatory attribute to None"

        # CHECK: Optional: 2
        op.optional_i32 = two
        print(f"Optional: {op.optional_i32.value}")

        # CHECK: Optional: None
        del op.optional_i32
        print(f"Optional: {op.optional_i32}")

        # CHECK: Unit: False
        op.unit = None
        print(f"Unit: {op.unit}")
        assert "unit" not in op.attributes

        # CHECK: Unit: True
        op.unit = True
        print(f"Unit: {op.unit}")

        # CHECK: Unit: False
        del op.unit
        print(f"Unit: {op.unit}")


# CHECK-LABEL: TEST: attrBuilder
@run
def attrBuilder():
    with Context() as ctx, Location.unknown():
        # CHECK: python_test.attributes_op
        op = test.AttributesOp(
            # CHECK-DAG: x_affinemap = affine_map<() -> (2)>
            x_affinemap=AffineMap.get_constant(2),
            # CHECK-DAG: x_affinemaparr = [affine_map<(d0, d1, d2) -> (d0, d1, d2)>]
            x_affinemaparr=[AffineMap.get_identity(3)],
            # CHECK-DAG: x_arr = [true, "x"]
            x_arr=[BoolAttr.get(True), StringAttr.get("x")],
            x_boolarr=[False, True],  # CHECK-DAG: x_boolarr = [false, true]
            x_bool=True,  # CHECK-DAG: x_bool = true
            x_dboolarr=[True, False],  # CHECK-DAG: x_dboolarr = array<i1: true, false>
            x_df16arr=[21, 22],  # CHECK-DAG: x_df16arr = array<i16: 21, 22>
            # CHECK-DAG: x_df32arr = array<f32: 2.300000e+01, 2.400000e+01>
            x_df32arr=[23, 24],
            # CHECK-DAG: x_df64arr = array<f64: 2.500000e+01, 2.600000e+01>
            x_df64arr=[25, 26],
            x_di32arr=[0, 1],  # CHECK-DAG: x_di32arr = array<i32: 0, 1>
            # CHECK-DAG: x_di64arr = array<i64: 1, 2>
            x_di64arr=[1, 2],
            x_di8arr=[2, 3],  # CHECK-DAG: x_di8arr = array<i8: 2, 3>
            # CHECK-DAG: x_dictarr = [{a = false}]
            x_dictarr=[{"a": BoolAttr.get(False)}],
            x_dict={"b": BoolAttr.get(True)},  # CHECK-DAG: x_dict = {b = true}
            x_f32=-2.25,  # CHECK-DAG: x_f32 = -2.250000e+00 : f32
            # CHECK-DAG: x_f32arr = [2.000000e+00 : f32, 3.000000e+00 : f32]
            x_f32arr=[2.0, 3.0],
            x_f64=4.25,  # CHECK-DAG: x_f64 = 4.250000e+00 : f64
            x_f64arr=[4.0, 8.0],  # CHECK-DAG: x_f64arr = [4.000000e+00, 8.000000e+00]
            # CHECK-DAG: x_f64elems = dense<[8.000000e+00, 1.600000e+01]> : tensor<2xf64>
            x_f64elems=[8.0, 16.0],
            # CHECK-DAG: x_flatsymrefarr = [@symbol1, @symbol2]
            x_flatsymrefarr=["symbol1", "symbol2"],
            x_flatsymref="symbol3",  # CHECK-DAG: x_flatsymref = @symbol3
            x_i1=0,  # CHECK-DAG: x_i1 = false
            x_i16=42,  # CHECK-DAG: x_i16 = 42 : i16
            x_i32=6,  # CHECK-DAG: x_i32 = 6 : i32
            x_i32arr=[4, 5],  # CHECK-DAG: x_i32arr = [4 : i32, 5 : i32]
            x_i32elems=[5, 6],  # CHECK-DAG: x_i32elems = dense<[5, 6]> : tensor<2xi32>
            x_i64=9,  # CHECK-DAG: x_i64 = 9 : i64
            x_i64arr=[7, 8],  # CHECK-DAG: x_i64arr = [7, 8]
            x_i64elems=[8, 9],  # CHECK-DAG: x_i64elems = dense<[8, 9]> : tensor<2xi64>
            x_i64svecarr=[10, 11],  # CHECK-DAG: x_i64svecarr = [10, 11]
            x_i8=11,  # CHECK-DAG: x_i8 = 11 : i8
            x_idx=10,  # CHECK-DAG: x_idx = 10 : index
            # CHECK-DAG: x_idxelems = dense<[11, 12]> : tensor<2xindex>
            x_idxelems=[11, 12],
            # CHECK-DAG: x_idxlistarr = [{{\[}}13], [14, 15]]
            x_idxlistarr=[[13], [14, 15]],
            x_si1=-1,  # CHECK-DAG: x_si1 = -1 : si1
            x_si16=-2,  # CHECK-DAG: x_si16 = -2 : si16
            x_si32=-3,  # CHECK-DAG: x_si32 = -3 : si32
            x_si64=-123,  # CHECK-DAG: x_si64 = -123 : si64
            x_si8=-4,  # CHECK-DAG: x_si8 = -4 : si8
            x_strarr=["hello", "world"],  # CHECK-DAG: x_strarr = ["hello", "world"]
            x_str="hello world!",  # CHECK-DAG: x_str = "hello world!"
            # CHECK-DAG: x_symrefarr = [@flatsym, @deep::@sym]
            x_symrefarr=["flatsym", ["deep", "sym"]],
            x_symref=["deep", "sym2"],  # CHECK-DAG: x_symref = @deep::@sym2
            x_sym="symbol",  # CHECK-DAG: x_sym = "symbol"
            x_typearr=[F32Type.get()],  # CHECK-DAG: x_typearr = [f32]
            x_type=F64Type.get(),  # CHECK-DAG: x_type = f64
            x_ui1=1,  # CHECK-DAG: x_ui1 = 1 : ui1
            x_ui16=2,  # CHECK-DAG: x_ui16 = 2 : ui16
            x_ui32=3,  # CHECK-DAG: x_ui32 = 3 : ui32
            x_ui64=4,  # CHECK-DAG: x_ui64 = 4 : ui64
            x_ui8=5,  # CHECK-DAG: x_ui8 = 5 : ui8
            x_unit=True,  # CHECK-DAG: x_unit
        )
        op.verify()
        op.print(use_local_scope=True)

    # fmt: off
    assert typing.get_type_hints(test.AttributesOp.x_affinemaparr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_affinemaparr.fget)["return"] is ArrayAttr
    assert type(op.x_affinemaparr) is typing.get_type_hints(test.AttributesOp.x_affinemaparr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_affinemap.fset)["value"] is AffineMapAttr
    assert typing.get_type_hints(test.AttributesOp.x_affinemap.fget)["return"] is AffineMapAttr
    assert type(op.x_affinemap) is typing.get_type_hints(test.AttributesOp.x_affinemap.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_arr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_arr.fget)["return"] is ArrayAttr
    assert type(op.x_arr) is typing.get_type_hints(test.AttributesOp.x_arr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_boolarr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_boolarr.fget)["return"] is ArrayAttr
    assert type(op.x_boolarr) is typing.get_type_hints(test.AttributesOp.x_boolarr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_bool.fset)["value"] is BoolAttr
    assert typing.get_type_hints(test.AttributesOp.x_bool.fget)["return"] is BoolAttr
    assert type(op.x_bool) is typing.get_type_hints(test.AttributesOp.x_bool.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_dboolarr.fset)["value"] is DenseBoolArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_dboolarr.fget)["return"] is DenseBoolArrayAttr
    assert type(op.x_dboolarr) is typing.get_type_hints(test.AttributesOp.x_dboolarr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_df32arr.fset)["value"] is DenseF32ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_df32arr.fget)["return"] is DenseF32ArrayAttr
    assert type(op.x_df32arr) is typing.get_type_hints(test.AttributesOp.x_df32arr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_df64arr.fset)["value"] is DenseF64ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_df64arr.fget)["return"] is DenseF64ArrayAttr
    assert type(op.x_df64arr) is typing.get_type_hints(test.AttributesOp.x_df64arr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_df16arr.fset)["value"] is DenseI16ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_df16arr.fget)["return"] is DenseI16ArrayAttr
    assert type(op.x_df16arr) is typing.get_type_hints(test.AttributesOp.x_df16arr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_di32arr.fset)["value"] is DenseI32ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_di32arr.fget)["return"] is DenseI32ArrayAttr
    assert type(op.x_di32arr) is typing.get_type_hints(test.AttributesOp.x_di32arr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_di64arr.fset)["value"] is DenseI64ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_di64arr.fget)["return"] is DenseI64ArrayAttr
    assert type(op.x_di64arr) is typing.get_type_hints(test.AttributesOp.x_di64arr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_di8arr.fset)["value"] is DenseI8ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_di8arr.fget)["return"] is DenseI8ArrayAttr
    assert type(op.x_di8arr) is typing.get_type_hints(test.AttributesOp.x_di8arr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_dictarr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_dictarr.fget)["return"] is ArrayAttr
    assert type(op.x_dictarr) is typing.get_type_hints(test.AttributesOp.x_dictarr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_dict.fset)["value"] is DictAttr
    assert typing.get_type_hints(test.AttributesOp.x_dict.fget)["return"] is DictAttr
    assert type(op.x_dict) is typing.get_type_hints(test.AttributesOp.x_dict.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_f32arr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_f32arr.fget)["return"] is ArrayAttr
    assert type(op.x_f32arr) is typing.get_type_hints(test.AttributesOp.x_f32arr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_f32.fset)["value"] is FloatAttr
    assert typing.get_type_hints(test.AttributesOp.x_f32.fget)["return"] is FloatAttr
    assert type(op.x_f32) is typing.get_type_hints(test.AttributesOp.x_f32.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_f64arr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_f64arr.fget)["return"] is ArrayAttr
    assert type(op.x_f64arr) is typing.get_type_hints(test.AttributesOp.x_f64arr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_f64.fset)["value"] is FloatAttr
    assert typing.get_type_hints(test.AttributesOp.x_f64.fget)["return"] is FloatAttr
    assert type(op.x_f64) is typing.get_type_hints(test.AttributesOp.x_f64.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_f64elems.fset)["value"] is DenseFPElementsAttr
    assert typing.get_type_hints(test.AttributesOp.x_f64elems.fget)["return"] is DenseFPElementsAttr
    assert type(op.x_f64elems) is typing.get_type_hints(test.AttributesOp.x_f64elems.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_flatsymrefarr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_flatsymrefarr.fget)["return"] is ArrayAttr
    assert type(op.x_flatsymrefarr) is typing.get_type_hints(test.AttributesOp.x_flatsymrefarr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_flatsymref.fset)["value"] is FlatSymbolRefAttr
    assert typing.get_type_hints(test.AttributesOp.x_flatsymref.fget)["return"] is FlatSymbolRefAttr
    assert type(op.x_flatsymref) is typing.get_type_hints(test.AttributesOp.x_flatsymref.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_i16.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_i16.fget)["return"] is IntegerAttr
    assert type(op.x_i16) is typing.get_type_hints(test.AttributesOp.x_i16.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_i1.fset)["value"] is BoolAttr
    assert typing.get_type_hints(test.AttributesOp.x_i1.fget)["return"] is BoolAttr
    assert type(op.x_i1) is typing.get_type_hints(test.AttributesOp.x_i1.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_i32arr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_i32arr.fget)["return"] is ArrayAttr
    assert type(op.x_i32arr) is typing.get_type_hints(test.AttributesOp.x_i32arr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_i32.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_i32.fget)["return"] is IntegerAttr
    assert type(op.x_i32) is typing.get_type_hints(test.AttributesOp.x_i32.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_i32elems.fset)["value"] is DenseIntElementsAttr
    assert typing.get_type_hints(test.AttributesOp.x_i32elems.fget)["return"] is DenseIntElementsAttr
    assert type(op.x_i32elems) is typing.get_type_hints(test.AttributesOp.x_i32elems.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_i64arr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_i64arr.fget)["return"] is ArrayAttr
    assert type(op.x_i64arr) is typing.get_type_hints(test.AttributesOp.x_i64arr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_i64.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_i64.fget)["return"] is IntegerAttr
    assert type(op.x_i64) is typing.get_type_hints(test.AttributesOp.x_i64.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_i64elems.fset)["value"] is DenseIntElementsAttr
    assert typing.get_type_hints(test.AttributesOp.x_i64elems.fget)["return"] is DenseIntElementsAttr
    assert type(op.x_i64elems) is typing.get_type_hints(test.AttributesOp.x_i64elems.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_i64svecarr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_i64svecarr.fget)["return"] is ArrayAttr
    assert type(op.x_i64svecarr) is typing.get_type_hints(test.AttributesOp.x_i64svecarr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_i8.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_i8.fget)["return"] is IntegerAttr
    assert type(op.x_i8) is typing.get_type_hints(test.AttributesOp.x_i8.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_idx.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_idx.fget)["return"] is IntegerAttr
    assert type(op.x_idx) is typing.get_type_hints(test.AttributesOp.x_idx.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_idxelems.fset)["value"] is DenseIntElementsAttr
    assert typing.get_type_hints(test.AttributesOp.x_idxelems.fget)["return"] is DenseIntElementsAttr
    assert type(op.x_idxelems) is typing.get_type_hints(test.AttributesOp.x_idxelems.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_idxlistarr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_idxlistarr.fget)["return"] is ArrayAttr
    assert type(op.x_idxlistarr) is typing.get_type_hints(test.AttributesOp.x_idxlistarr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_si16.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_si16.fget)["return"] is IntegerAttr
    assert type(op.x_si16) is typing.get_type_hints(test.AttributesOp.x_si16.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_si1.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_si1.fget)["return"] is IntegerAttr
    assert type(op.x_si1) is typing.get_type_hints(test.AttributesOp.x_si1.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_si32.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_si32.fget)["return"] is IntegerAttr
    assert type(op.x_si32) is typing.get_type_hints(test.AttributesOp.x_si32.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_si64.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_si64.fget)["return"] is IntegerAttr
    assert type(op.x_si64) is typing.get_type_hints(test.AttributesOp.x_si64.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_si8.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_si8.fget)["return"] is IntegerAttr
    assert type(op.x_si8) is typing.get_type_hints(test.AttributesOp.x_si8.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_strarr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_strarr.fget)["return"] is ArrayAttr
    assert type(op.x_strarr) is typing.get_type_hints(test.AttributesOp.x_strarr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_str.fset)["value"] is StringAttr
    assert typing.get_type_hints(test.AttributesOp.x_str.fget)["return"] is StringAttr
    assert type(op.x_str) is typing.get_type_hints(test.AttributesOp.x_str.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_sym.fset)["value"] is StringAttr
    assert typing.get_type_hints(test.AttributesOp.x_sym.fget)["return"] is StringAttr
    assert type(op.x_sym) is typing.get_type_hints(test.AttributesOp.x_sym.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_symrefarr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_symrefarr.fget)["return"] is ArrayAttr
    assert type(op.x_symrefarr) is typing.get_type_hints(test.AttributesOp.x_symrefarr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_symref.fset)["value"] is SymbolRefAttr
    assert typing.get_type_hints(test.AttributesOp.x_symref.fget)["return"] is SymbolRefAttr
    assert type(op.x_symref) is typing.get_type_hints(test.AttributesOp.x_symref.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_typearr.fset)["value"] is ArrayAttr
    assert typing.get_type_hints(test.AttributesOp.x_typearr.fget)["return"] is ArrayAttr
    assert type(op.x_typearr) is typing.get_type_hints(test.AttributesOp.x_typearr.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_type.fset)["value"] is TypeAttr
    assert typing.get_type_hints(test.AttributesOp.x_type.fget)["return"] is TypeAttr
    assert type(op.x_type) is typing.get_type_hints(test.AttributesOp.x_type.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_ui16.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_ui16.fget)["return"] is IntegerAttr
    assert type(op.x_ui16) is typing.get_type_hints(test.AttributesOp.x_ui16.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_ui1.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_ui1.fget)["return"] is IntegerAttr
    assert type(op.x_ui1) is typing.get_type_hints(test.AttributesOp.x_ui1.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_ui32.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_ui32.fget)["return"] is IntegerAttr
    assert type(op.x_ui32) is typing.get_type_hints(test.AttributesOp.x_ui32.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_ui64.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_ui64.fget)["return"] is IntegerAttr
    assert type(op.x_ui64) is typing.get_type_hints(test.AttributesOp.x_ui64.fget)["return"]

    assert typing.get_type_hints(test.AttributesOp.x_ui8.fset)["value"] is IntegerAttr
    assert typing.get_type_hints(test.AttributesOp.x_ui8.fget)["return"] is IntegerAttr
    assert type(op.x_ui8) is typing.get_type_hints(test.AttributesOp.x_ui8.fget)["return"]
    # fmt: on


# CHECK-LABEL: TEST: inferReturnTypes
@run
def inferReturnTypes():
    with Context() as ctx, Location.unknown(ctx):
        module = Module.create()
        with InsertionPoint(module.body):
            op = test.InferResultsOp()
            dummy = test.DummyOp()

        # CHECK: [Type(i32), Type(i64)]
        iface = InferTypeOpInterface(op)
        print(iface.inferReturnTypes())

        # CHECK: [Type(i32), Type(i64)]
        iface_static = InferTypeOpInterface(test.InferResultsOp)
        print(iface.inferReturnTypes())

        assert isinstance(iface.opview, test.InferResultsOp)
        assert iface.opview == iface.operation.opview

        try:
            iface_static.opview
        except TypeError:
            pass
        else:
            assert False, (
                "not expected to be able to obtain an opview from a static" " interface"
            )

        try:
            InferTypeOpInterface(dummy)
        except ValueError:
            pass
        else:
            assert False, "not expected dummy op to implement the interface"

        try:
            InferTypeOpInterface(test.DummyOp)
        except ValueError:
            pass
        else:
            assert False, "not expected dummy op class to implement the interface"


# CHECK-LABEL: TEST: resultTypesDefinedByTraits
@run
def resultTypesDefinedByTraits():
    with Context() as ctx, Location.unknown(ctx):
        module = Module.create()
        with InsertionPoint(module.body):
            inferred = test.InferResultsOp()

            # CHECK: i32 i64
            print(inferred.single.type, inferred.doubled.type)

            same = test.SameOperandAndResultTypeOp([inferred.results[0]])
            # CHECK-COUNT-2: i32
            print(same.one.type)
            print(same.two.type)
            assert (
                typing.get_type_hints(test.SameOperandAndResultTypeOp.one.fget)[
                    "return"
                ]
                is OpResult
            )
            assert type(same.one) is OpResult

            first_type_attr = test.FirstAttrDeriveTypeAttrOp(
                inferred.results[1], TypeAttr.get(IndexType.get())
            )
            # CHECK-COUNT-2: index
            print(first_type_attr.one.type)
            print(first_type_attr.two.type)

            first_attr = test.FirstAttrDeriveAttrOp(FloatAttr.get(F32Type.get(), 3.14))
            # CHECK-COUNT-3: f32
            print(first_attr.one.type)
            print(first_attr.two.type)
            print(first_attr.three.type)

            implied = test.InferResultsImpliedOp()
            # CHECK: i32
            print(implied.integer.type)
            # CHECK: f64
            print(implied.flt.type)
            # CHECK: index
            print(implied.index.type)

            # provide the result types to avoid inferring them
            f64 = F64Type.get()
            no_imply = test.InferResultsImpliedOp(results=[f64, f64, f64])
            # CHECK-COUNT-3: f64
            print(no_imply.integer.type, no_imply.flt.type, no_imply.index.type)

            no_infer = test.InferResultsOp(results=[F32Type.get(), IndexType.get()])
            # CHECK: f32 index
            print(no_infer.single.type, no_infer.doubled.type)


# CHECK-LABEL: TEST: testOptionalOperandOp
@run
def testOptionalOperandOp():
    with Context() as ctx, Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            op1 = test.OptionalOperandOp()
            # CHECK: op1.input is None: True
            print(f"op1.input is None: {op1.input is None}")
            assert (
                typing.get_type_hints(test.OptionalOperandOp.input.fget)["return"]
                is Optional[Value]
            )
            assert (
                typing.get_type_hints(test.OptionalOperandOp.result.fget)["return"]
                == OpResult[IntegerType]
            )
            assert type(op1.result) is OpResult

            op2 = test.OptionalOperandOp(input=op1)
            # CHECK: op2.input is None: False
            print(f"op2.input is None: {op2.input is None}")


# CHECK-LABEL: TEST: testCustomAttribute
@run
def testCustomAttribute():
    with Context() as ctx, Location.unknown():
        a = TestAttr.get()
        # CHECK: #python_test.test_attr
        print(a)

        # CHECK: python_test.custom_attributed_op  {
        # CHECK: #python_test.test_attr
        # CHECK: }
        op2 = test.CustomAttributedOp(a)
        print(f"{op2}")

        # CHECK: #python_test.test_attr
        print(f"{op2.test_attr}")

        # CHECK: TestAttr(#python_test.test_attr)
        print(repr(op2.test_attr))

        # The following cast must not assert.
        b = TestAttr(a)

        unit = UnitAttr.get()
        try:
            TestAttr(unit)
        except ValueError as e:
            assert "Cannot cast attribute to TestAttr" in str(e)
        else:
            raise

        # The following must trigger a TypeError from our adaptors and must not
        # crash.
        try:
            TestAttr(42)
        except TypeError as e:
            assert (
                "__init__(): incompatible function arguments. The following argument types are supported"
                in str(e)
            )
            assert (
                "__init__(self, cast_from_attr: mlir._mlir_libs._mlir.ir.Attribute) -> None"
                in str(e)
            )
            assert (
                "Invoked with types: mlir._mlir_libs._mlirPythonTestNanobind.TestAttr, int"
                in str(e)
            )
        else:
            raise

        # The following must trigger a TypeError from pybind (therefore, not
        # checking its message) and must not crash.
        try:
            TestAttr(42, 56)
        except TypeError:
            pass
        else:
            raise


@run
def testCustomType():
    with Context() as ctx:
        a = TestType.get()
        # CHECK: !python_test.test_type
        print(a)

        # The following cast must not assert.
        b = TestType(a)
        # Instance custom types should have typeids
        assert isinstance(b.typeid, TypeID)

        i8 = IntegerType.get_signless(8)
        try:
            TestType(i8)
        except ValueError as e:
            assert "Cannot cast type to TestType" in str(e)
        else:
            raise

        # The following must trigger a TypeError from our adaptors and must not
        # crash.
        try:
            TestType(42)
        except TypeError as e:
            assert (
                "__init__(): incompatible function arguments. The following argument types are supported"
                in str(e)
            )
            assert (
                "__init__(self, cast_from_type: mlir._mlir_libs._mlir.ir.Type) -> None"
                in str(e)
            )
            assert (
                "Invoked with types: mlir._mlir_libs._mlirPythonTestNanobind.TestType, int"
                in str(e)
            )
        else:
            raise

        # The following must trigger a TypeError from pybind (therefore, not
        # checking its message) and must not crash.
        try:
            TestType(42, 56)
        except TypeError:
            pass
        else:
            raise


@run
# CHECK-LABEL: TEST: testValue
def testValue():
    # Check that Value is a generic class at runtime.
    assert hasattr(Value, "__class_getitem__")


@run
# CHECK-LABEL: TEST: testTensorValue
def testTensorValue():
    with Context() as ctx, Location.unknown():
        i8 = IntegerType.get_signless(8)

        class Tensor(TestTensorValue):
            def __str__(self):
                return super().__str__().replace("Value", "Tensor")

        module = Module.create()
        with InsertionPoint(module.body):
            t = tensor.EmptyOp([10, 10], i8).result

            # CHECK: Value(%{{.*}} = tensor.empty() : tensor<10x10xi8>)
            print(Value(t))

            tt = Tensor(t)
            # CHECK: Tensor(%{{.*}} = tensor.empty() : tensor<10x10xi8>)
            print(tt)

            # CHECK: False
            print(tt.is_null())

            # Classes of custom types that inherit from concrete types should have
            # static_typeid
            assert isinstance(TestIntegerRankedTensorType.static_typeid, TypeID)
            # And it should be equal to the in-tree concrete type
            assert TestIntegerRankedTensorType.static_typeid == t.type.typeid

            d = tensor.EmptyOp([1, 2, 3], IntegerType.get_signless(5)).result
            # CHECK: Value(%{{.*}} = tensor.empty() : tensor<1x2x3xi5>)
            print(d)
            # CHECK: TestTensorValue
            print(repr(d))


# CHECK-LABEL: TEST: inferReturnTypeComponents
@run
def inferReturnTypeComponents():
    with Context() as ctx, Location.unknown(ctx):
        module = Module.create()
        i32 = IntegerType.get_signless(32)
        with InsertionPoint(module.body):
            resultType = UnrankedTensorType.get(i32)
            operandTypes = [
                RankedTensorType.get([1, 3, 10, 10], i32),
                UnrankedTensorType.get(i32),
            ]
            f = func.FuncOp(
                "test_inferReturnTypeComponents", (operandTypes, [resultType])
            )
            entry_block = Block.create_at_start(f.operation.regions[0], operandTypes)
            with InsertionPoint(entry_block):
                ranked_op = test.InferShapedTypeComponentsOp(
                    resultType, entry_block.arguments[0]
                )
                unranked_op = test.InferShapedTypeComponentsOp(
                    resultType, entry_block.arguments[1]
                )

        # CHECK: has rank: True
        # CHECK: rank: 4
        # CHECK: element type: i32
        # CHECK: shape: [1, 3, 10, 10]
        iface = InferShapedTypeOpInterface(ranked_op)
        shaped_type_components = iface.inferReturnTypeComponents(
            operands=[ranked_op.operand]
        )[0]
        print("has rank:", shaped_type_components.has_rank)
        print("rank:", shaped_type_components.rank)
        print("element type:", shaped_type_components.element_type)
        print("shape:", shaped_type_components.shape)

        # CHECK: has rank: False
        # CHECK: rank: None
        # CHECK: element type: i32
        # CHECK: shape: None
        iface = InferShapedTypeOpInterface(unranked_op)
        shaped_type_components = iface.inferReturnTypeComponents(
            operands=[unranked_op.operand]
        )[0]
        print("has rank:", shaped_type_components.has_rank)
        print("rank:", shaped_type_components.rank)
        print("element type:", shaped_type_components.element_type)
        print("shape:", shaped_type_components.shape)


# CHECK-LABEL: TEST: testCustomTypeTypeCaster
@run
def testCustomTypeTypeCaster():
    with Context() as ctx, Location.unknown():
        a = TestType.get()
        assert a.typeid is not None

        b = Type.parse("!python_test.test_type")
        # CHECK: !python_test.test_type
        print(b)
        # CHECK: TestType(!python_test.test_type)
        print(repr(b))

        c = TestIntegerRankedTensorType.get([10, 10], 5)
        # CHECK: tensor<10x10xi5>
        print(c)
        # CHECK: TestIntegerRankedTensorType(tensor<10x10xi5>)
        print(repr(c))

        # CHECK: Type caster is already registered
        try:

            @register_type_caster(c.typeid)
            def type_caster(pytype):
                return TestIntegerRankedTensorType(pytype)

        except RuntimeError as e:
            print(e)

        # python_test dialect registers a caster for RankedTensorType in its extension (pybind) module.
        # So this one replaces that one (successfully). And then just to be sure we restore the original caster below.
        @register_type_caster(c.typeid, replace=True)
        def type_caster(pytype):
            return RankedTensorType(pytype)

        d = tensor.EmptyOp([10, 10], IntegerType.get_signless(5)).result
        # CHECK: tensor<10x10xi5>
        print(d.type)
        # CHECK: ranked tensor type RankedTensorType(tensor<10x10xi5>)
        print("ranked tensor type", repr(d.type))

        @register_type_caster(c.typeid, replace=True)
        def type_caster(pytype):
            return TestIntegerRankedTensorType(pytype)

        d = tensor.EmptyOp([10, 10], IntegerType.get_signless(5)).result
        # CHECK: tensor<10x10xi5>
        print(d.type)
        # CHECK: TestIntegerRankedTensorType(tensor<10x10xi5>)
        print(repr(d.type))


# CHECK-LABEL: TEST: testInferTypeOpInterface
@run
def testInferTypeOpInterface():
    with Context() as ctx, Location.unknown(ctx):
        module = Module.create()
        with InsertionPoint(module.body):
            i64 = IntegerType.get_signless(64)
            zero = arith.ConstantOp(i64, 0)

            one_operand = test.InferResultsVariadicInputsOp(single=zero, doubled=None)
            # CHECK: i32
            print(one_operand.result.type)

            two_operands = test.InferResultsVariadicInputsOp(single=zero, doubled=zero)
            # CHECK: f32
            print(two_operands.result.type)

            assert (
                typing.get_type_hints(test.infer_results_variadic_inputs_op)["return"]
                is OpResult
            )
            assert (
                type(test.infer_results_variadic_inputs_op(single=zero, doubled=zero))
                is OpResult
            )


# CHECK-LABEL: TEST: testVariadicOperandAccess
@run
def testVariadicOperandAccess():
    def values(lst):
        return [str(e) for e in lst]

    with Context() as ctx, Location.unknown(ctx):
        module = Module.create()
        with InsertionPoint(module.body):
            i32 = IntegerType.get_signless(32)
            zero = arith.ConstantOp(i32, 0)
            one = arith.ConstantOp(i32, 1)
            two = arith.ConstantOp(i32, 2)
            three = arith.ConstantOp(i32, 3)
            four = arith.ConstantOp(i32, 4)

            variadic_operands = test.SameVariadicOperandSizeOp(
                [zero, one], two, [three, four]
            )
            # CHECK: OpResult(%{{.*}} = arith.constant 2 : i32)
            print(variadic_operands.non_variadic)
            assert (
                typing.get_type_hints(test.SameVariadicOperandSizeOp.non_variadic.fget)[
                    "return"
                ]
                is Value
            )
            assert type(variadic_operands.non_variadic) is OpResult

            # CHECK: ['OpResult(%{{.*}} = arith.constant 0 : i32)', 'OpResult(%{{.*}} = arith.constant 1 : i32)']
            print(values(variadic_operands.variadic1))
            assert (
                typing.get_type_hints(test.SameVariadicOperandSizeOp.variadic1.fget)[
                    "return"
                ]
                is OpOperandList
            )
            assert type(variadic_operands.variadic1) is OpOperandList

            # CHECK: ['OpResult(%{{.*}} = arith.constant 3 : i32)', 'OpResult(%{{.*}} = arith.constant 4 : i32)']
            print(values(variadic_operands.variadic2))
            assert type(variadic_operands.variadic2) is OpOperandList

            assert (
                typing.get_type_hints(test.same_variadic_operand)["return"]
                is test.SameVariadicOperandSizeOp
            )
            assert (
                type(test.same_variadic_operand([zero, one], two, [three, four]))
                is test.SameVariadicOperandSizeOp
            )


# CHECK-LABEL: TEST: testVariadicResultAccess
@run
def testVariadicResultAccess():
    def types(lst):
        return [e.type for e in lst]

    with Context() as ctx, Location.unknown(ctx):
        module = Module.create()
        with InsertionPoint(module.body):
            i = [IntegerType.get_signless(k) for k in range(7)]

            # Test Variadic-Fixed-Variadic
            op = test.SameVariadicResultSizeOpVFV([i[0], i[1]], i[2], [i[3], i[4]])
            # CHECK: i2
            print(op.non_variadic.type)
            # CHECK: [IntegerType(i0), IntegerType(i1)]
            print(types(op.variadic1))
            # CHECK: [IntegerType(i3), IntegerType(i4)]
            print(types(op.variadic2))

            assert (
                typing.get_type_hints(test.same_variadic_result_vfv)["return"]
                == Union[OpResult, OpResultList, test.SameVariadicResultSizeOpVFV]
            )
            assert (
                type(test.same_variadic_result_vfv([i[0], i[1]], i[2], [i[3], i[4]]))
                is OpResultList
            )

            #  Test Variadic-Variadic-Variadic
            op = test.SameVariadicResultSizeOpVVV(
                [i[0], i[1]], [i[2], i[3]], [i[4], i[5]]
            )
            # CHECK: [IntegerType(i0), IntegerType(i1)]
            print(types(op.variadic1))
            # CHECK: [IntegerType(i2), IntegerType(i3)]
            print(types(op.variadic2))
            # CHECK: [IntegerType(i4), IntegerType(i5)]
            print(types(op.variadic3))

            #  Test Fixed-Fixed-Variadic
            op = test.SameVariadicResultSizeOpFFV(i[0], i[1], [i[2], i[3], i[4]])
            # CHECK: i0
            print(op.non_variadic1.type)
            # CHECK: i1
            print(op.non_variadic2.type)
            # CHECK: [IntegerType(i2), IntegerType(i3), IntegerType(i4)]
            print(types(op.variadic))
            assert (
                typing.get_type_hints(test.SameVariadicResultSizeOpFFV.variadic.fget)[
                    "return"
                ]
                is OpResultList
            )
            assert type(op.variadic) is OpResultList

            #  Test Variadic-Variadic-Fixed
            op = test.SameVariadicResultSizeOpVVF(
                [i[0], i[1], i[2]], [i[3], i[4], i[5]], i[6]
            )
            # CHECK: [IntegerType(i0), IntegerType(i1), IntegerType(i2)]
            print(types(op.variadic1))
            # CHECK: [IntegerType(i3), IntegerType(i4), IntegerType(i5)]
            print(types(op.variadic2))
            # CHECK: i6
            print(op.non_variadic.type)

            # Test Fixed-Variadic-Fixed-Variadic-Fixed
            op = test.SameVariadicResultSizeOpFVFVF(
                i[0], [i[1], i[2]], i[3], [i[4], i[5]], i[6]
            )
            # CHECK: i0
            print(op.non_variadic1.type)
            # CHECK: [IntegerType(i1), IntegerType(i2)]
            print(types(op.variadic1))
            # CHECK: i3
            print(op.non_variadic2.type)
            # CHECK: [IntegerType(i4), IntegerType(i5)]
            print(types(op.variadic2))
            # CHECK: i6
            print(op.non_variadic3.type)

            # Test Fixed-Variadic-Fixed-Variadic-Fixed - Variadic group size 0
            op = test.SameVariadicResultSizeOpFVFVF(i[0], [], i[1], [], i[2])
            # CHECK: i0
            print(op.non_variadic1.type)
            # CHECK: []
            print(types(op.variadic1))
            # CHECK: i1
            print(op.non_variadic2.type)
            # CHECK: []
            print(types(op.variadic2))
            # CHECK: i2
            print(op.non_variadic3.type)

            # Test Fixed-Variadic-Fixed-Variadic-Fixed - Variadic group size 1
            op = test.SameVariadicResultSizeOpFVFVF(i[0], [i[1]], i[2], [i[3]], i[4])
            # CHECK: i0
            print(op.non_variadic1.type)
            # CHECK: [IntegerType(i1)]
            print(types(op.variadic1))
            # CHECK: i2
            print(op.non_variadic2.type)
            # CHECK: [IntegerType(i3)]
            print(types(op.variadic2))
            # CHECK: i4
            print(op.non_variadic3.type)

            assert (
                typing.get_type_hints(test.results_variadic)["return"]
                == Union[OpResult, OpResultList, test.ResultsVariadicOp]
            )
            assert type(test.results_variadic([i[0]])) is OpResult
            op_res_variadic = test.ResultsVariadicOp([i[0]])
            assert (
                typing.get_type_hints(test.ResultsVariadicOp.res.fget)["return"]
                is OpResultList
            )
            assert type(op_res_variadic.res) is OpResultList


# CHECK-LABEL: TEST: testVariadicAndNormalRegionOp
@run
def testVariadicAndNormalRegionOp():
    with Context() as ctx, Location.unknown(ctx):
        module = Module.create()
        with InsertionPoint(module.body):
            region_op = test.VariadicAndNormalRegionOp(2)
            assert (
                typing.get_type_hints(test.VariadicAndNormalRegionOp.region.fget)[
                    "return"
                ]
                is Region
            )
            assert type(region_op.region) is Region
            assert (
                typing.get_type_hints(test.VariadicAndNormalRegionOp.variadic.fget)[
                    "return"
                ]
                is RegionSequence
            )
            assert type(region_op.variadic) is RegionSequence

            assert isinstance(region_op.opview, OpView)
            assert isinstance(region_op.operation.opview, OpView)



"""
Test SBSymbolContext APIs.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SymbolContextTwoFilesTestCase(TestBase):
    @expectedFailureAll(oslist=["windows"])
    def test_lookup_by_address(self):
        """Test lookup by address in a module with multiple compilation units"""
        self.build()
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        module = target.GetModuleAtIndex(0)
        self.assertTrue(module.IsValid())
        for symbol_name in ["struct1::f()", "struct2::f()"]:
            sc_list = module.FindFunctions(symbol_name, lldb.eSymbolTypeCode)
            self.assertTrue(1, sc_list.GetSize())
            symbol_address = sc_list.GetContextAtIndex(0).GetSymbol().GetStartAddress()
            self.assertTrue(symbol_address.IsValid())
            sc_by_address = module.ResolveSymbolContextForAddress(
                symbol_address, lldb.eSymbolContextFunction
            )
            self.assertEqual(symbol_name, sc_by_address.GetFunction().GetName())

    def test_ranges_in_multiple_compile_unit(self):
        """This test verifies that we correctly handle the case when multiple
        compile unit contains DW_AT_ranges and DW_AT_ranges_base attributes."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        source1 = "file1.cpp"
        line1 = line_number(source1, "// Break1")
        breakpoint1 = target.BreakpointCreateByLocation(source1, line1)
        self.assertIsNotNone(breakpoint1)
        self.assertTrue(breakpoint1.IsValid())

        source2 = "file2.cpp"
        line2 = line_number(source2, "// Break2")
        breakpoint2 = target.BreakpointCreateByLocation(source2, line2)
        self.assertIsNotNone(breakpoint2)
        self.assertTrue(breakpoint2.IsValid())

        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertIsNotNone(process, PROCESS_IS_VALID)

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, breakpoint2)
        self.assertEqual(len(threads), 1)
        frame = threads[0].GetFrameAtIndex(0)
        value = frame.FindVariable("x")
        self.assertTrue(value.IsValid())

        process.Continue()

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, breakpoint1)
        self.assertEqual(len(threads), 1)
        frame = threads[0].GetFrameAtIndex(0)
        value = frame.FindVariable("x")
        self.assertTrue(value.IsValid())

        process.Continue()
